summaryrefslogtreecommitdiff
path: root/sw/source/core/layout/sectfrm.cxx
diff options
context:
space:
mode:
authorMiklos Vajna <vmiklos@collabora.co.uk>2017-06-19 15:24:12 +0200
committerMiklos Vajna <vmiklos@collabora.co.uk>2017-06-19 20:26:27 +0200
commitf991b842addddeada6dc45c4054deeca5aa7f17b (patch)
tree47df98825dc411c984be83c20d35600cd8ad0b99 /sw/source/core/layout/sectfrm.cxx
parentecf88f83f877fd08a6d5a418c17fdf36b82a8369 (diff)
tdf#108524 sw: attempt to split section frames inside table cells
Tables-in-sections were already split across multiple pages, but not secions-in-tables. To be safe still don't allow sections-in-tables-in-sections, so you can combine these in both orders now, but not recursively. To achieve this, relax two "not in table" conditions to just require "not in a table that is already in a section", and define that in case a section-in-table is to be split, the follow section frame should be inserted under the follow of its cell. With this, finally the section frame in the bugdoc is split into two, and the second section frame is moved to the second page as expected. Change-Id: I16ebb2d30870b145a2378d46603324ab267b0dd3 Reviewed-on: https://gerrit.libreoffice.org/38965 Reviewed-by: Miklos Vajna <vmiklos@collabora.co.uk> Tested-by: Jenkins <ci@libreoffice.org>
Diffstat (limited to 'sw/source/core/layout/sectfrm.cxx')
-rw-r--r--sw/source/core/layout/sectfrm.cxx32
1 files changed, 28 insertions, 4 deletions
diff --git a/sw/source/core/layout/sectfrm.cxx b/sw/source/core/layout/sectfrm.cxx
index b94ba4582d85..06c186c5d4c2 100644
--- a/sw/source/core/layout/sectfrm.cxx
+++ b/sw/source/core/layout/sectfrm.cxx
@@ -22,6 +22,7 @@
#include <fmtftn.hxx>
#include <fmtclbl.hxx>
#include "sectfrm.hxx"
+#include "cellfrm.hxx"
#include "section.hxx"
#include <IDocumentSettingAccess.hxx>
#include "rootfrm.hxx"
@@ -587,6 +588,16 @@ namespace
return pLayFrame->GetNextLayoutLeaf();
return pLayFrame;
}
+
+ /// Checks if pFrame is in a table, which itself is in a section.
+ bool IsInTableInSection(SwFrame* pFrame)
+ {
+ if (!pFrame->IsInTab())
+ return false;
+
+ // The frame is in a table, see if the table is in a section.
+ return pFrame->FindTabFrame()->IsInSct();
+ }
}
void SwSectionFrame::MoveContentAndDelete( SwSectionFrame* pDel, bool bSave )
@@ -1439,9 +1450,9 @@ SwLayoutFrame *SwFrame::GetNextSctLeaf( MakePageType eMakePage )
return static_cast<SwLayoutFrame*>(static_cast<SwLayoutFrame*>(GetUpper()->GetNext())->Lower());
if( GetUpper()->IsColBodyFrame() && GetUpper()->GetUpper()->GetNext() )
return static_cast<SwLayoutFrame*>(static_cast<SwLayoutFrame*>(GetUpper()->GetUpper()->GetNext())->Lower());
// Inside a section, in tables, or sections of headers/footers, there can be only
// one column shift be made, one of the above shortcuts should have applied!
- if( GetUpper()->IsInTab() || FindFooterOrHeader() )
+ if( IsInTableInSection(GetUpper()) || FindFooterOrHeader() )
return nullptr;
SwSectionFrame *pSect = FindSctFrame();
@@ -1498,6 +1509,9 @@ SwLayoutFrame *SwFrame::GetNextSctLeaf( MakePageType eMakePage )
const bool bBody = IsInDocBody();
const bool bFootnotePage = FindPageFrame()->IsFootnotePage();
+ // The "pLayLeaf is in a table" case is rejected by default, so that it
+ // can't happen that we try to move a table to one of its own cells.
+ bool bLayLeafTableAllowed = false;
SwLayoutFrame *pLayLeaf;
// A shortcut for TabFrames such that not all cells need to be visited
if( bWrongPage )
@@ -1507,6 +1521,16 @@ SwLayoutFrame *SwFrame::GetNextSctLeaf( MakePageType eMakePage )
SwContentFrame* pTmpCnt = static_cast<SwTabFrame*>(this)->FindLastContent();
pLayLeaf = pTmpCnt ? pTmpCnt->GetUpper() : nullptr;
}
+ else if (IsInTab() && !IsInTableInSection(this))
+ {
+ // This frame is in a table-not-in-section, its follow should be
+ // inserted under the follow of the frame's cell.
+ pLayLeaf = FindCellFrame()->GetFollowCell();
+ if (pLayLeaf->FindTabFrame() == FindTabFrame())
+ SAL_WARN("sw.layout", "my table frame and my follow's table frame is the same");
+ // In this case pLayLeaf pointing to an in-table frame is OK.
+ bLayLeafTableAllowed = true;
+ }
else
{
pLayLeaf = GetNextLayoutLeaf();
@@ -1534,10 +1558,10 @@ SwLayoutFrame *SwFrame::GetNextSctLeaf( MakePageType eMakePage )
pLayLeaf = nullptr;
continue;
}
- // Once inBody always inBody, don't step into tables and not into other sections
+ // Once inBody always inBody, don't step into tables-in-sections and not into other sections
if ( (bBody && !pLayLeaf->IsInDocBody()) ||
(IsInFootnote() != pLayLeaf->IsInFootnote() ) ||
- pLayLeaf->IsInTab() ||
+ (pLayLeaf->IsInTab() && !bLayLeafTableAllowed) ||
( pLayLeaf->IsInSct() && ( !pSect->HasFollow()
|| pSect->GetFollow() != pLayLeaf->FindSctFrame() ) ) )
{
@@ -1439,9 +1450,9 @@ SwLayoutFrame *SwFrame::GetNextSctLeaf( MakePageType eMakePage )
if( GetUpper()->IsColBodyFrame() && GetUpper()->GetUpper()->GetNext() )
return static_cast<SwLayoutFrame*>(static_cast<SwLayoutFrame*>(GetUpper()->GetUpper()->GetNext())->Lower());
- // Inside a section, in tables, or sections of headers/footers, there can be only
+ // Inside a table-in-section, or sections of headers/footers, there can be only
// one column shift be made, one of the above shortcuts should have applied!